package be.iminds.ilabt.jfed.restapi;

import be.iminds.ilabt.jfed.lowlevel.AnyCredential;
import be.iminds.ilabt.jfed.lowlevel.CredentialException;
import be.iminds.ilabt.jfed.lowlevel.SimpleGeniUserWithSpeaksFor;
import be.iminds.ilabt.jfed.lowlevel.userloginmodel.InvalidLoginException;
import be.iminds.ilabt.jfed.restapi.jfedclient.JFedExperimenterApi;
import be.iminds.ilabt.jfed.restapi.jfedclient.JFedExperimenterApiFactory;
import be.iminds.ilabt.jfed.restapi.jfedclient.JFedExperimenterApiImpl;
import be.iminds.ilabt.jfed.restapi.jfedclient.MockJFedExperimenterApi;
import be.iminds.ilabt.jfed.restapi.resource.SpeakForCredentialParser;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import io.dropwizard.setup.Environment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * JFedRestApiModule
 */
public class JFedRestApiTestModule extends AbstractModule {
    private static final Logger LOG = LoggerFactory.getLogger(JFedRestApiTestModule.class);

    private final JFedRestApiConfiguration conf;
    private final Environment environment;


    public JFedRestApiTestModule(JFedRestApiConfiguration conf, Environment environment) {
        this.conf = conf;
        this.environment = environment;
        LOG.info("Constructing JFedRestApiTestModule");
    }

    @Override
    protected void configure() {
        bind(JFedRestApiConfiguration.class).toInstance(conf);
        bind(Environment.class).toInstance(environment);
    }

    @Provides
    public JFedExperimenterApiFactory providesJFedExperimenterApiFactory(JFedRestApiConfiguration conf) {
        return new JFedExperimenterApiFactory() {
            @Override
            public JFedExperimenterApi create(AnyCredential speaksForCredential, GeniUrn speakingFor)
                    throws SimpleGeniUserWithSpeaksFor.InvalidSpeaksForException, InvalidLoginException, IOException {
                try {
                    return new MockJFedExperimenterApi();
                } catch (Exception e) {
                    throw new RuntimeException("Error creating MockJFedExperimenterApi for test", e);
                }
            }
        };
    }

    @Provides
    public SpeakForCredentialParser providesSpeaksForCredentialParser() {
        return new SpeakForCredentialParser() {
            @Override
            public AnyCredential parse(String comment, String speaksForCredentialString) throws CredentialException {
                AnyCredential mockAnyCredential = mock(AnyCredential.class);
                when(mockAnyCredential.isSpeaksFor()).thenReturn(true);
                return mockAnyCredential;
            }
        };
    }
}
